<?php
/**
 * Plugin Name: Genex Detach & Restore Categories
 * Plugin URI: https://genexmarketingagency.com/
 * Description: Detach all products from their categories with automatic backup and restore functionality.
 * Version: 1.0.1
 * Author: Genex Marketing Agency Ltd.
 * Author URI: https://genexmarketingagency.com/
 * Text Domain: genex-detach-restore
 * WC requires at least: 3.0.0
 * WC tested up to: 8.0
 */

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit;
}

// Check if WooCommerce is active
if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
    add_action('admin_notices', function() {
        echo '<div class="error"><p>' . __('Genex Detach & Restore Categories requires WooCommerce to be installed and active.', 'genex-detach-restore') . '</p></div>';
    });
    return;
}

class Genex_Detach_Restore_Categories {
    
    // Plugin version
    private $version = '1.0.1';
    
    // Max number of backups to keep
    private $max_backups = 5;
    
    // Singleton instance
    private static $instance = null;
    
    // Get instance
    public static function get_instance() {
        if (null === self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    
    // Constructor
    public function __construct() {
        // Add admin menu
        add_action('admin_menu', array($this, 'add_admin_menu'));
        
        // Register AJAX handlers
        add_action('wp_ajax_genex_backup_category_relations', array($this, 'ajax_backup_category_relations'));
        add_action('wp_ajax_genex_detach_products', array($this, 'ajax_detach_products'));
        add_action('wp_ajax_genex_restore_backup', array($this, 'ajax_restore_backup'));
        add_action('wp_ajax_genex_fix_category_counts', array($this, 'ajax_fix_category_counts'));
        
        // Enqueue scripts and styles
        add_action('admin_enqueue_scripts', array($this, 'enqueue_scripts'));
    }
    
    // Add admin menu
    public function add_admin_menu() {
        add_submenu_page(
            'edit.php?post_type=product',
            __('Detach & Restore Categories', 'genex-detach-restore'),
            __('Detach & Restore Categories', 'genex-detach-restore'),
            'manage_woocommerce',
            'genex-detach-restore',
            array($this, 'admin_page')
        );
    }
    
    // Enqueue scripts and styles
    public function enqueue_scripts($hook) {
        if ('product_page_genex-detach-restore' !== $hook) {
            return;
        }
        
        wp_enqueue_style('genex-detach-restore-style', plugin_dir_url(__FILE__) . 'css/admin.css', array(), $this->version);
        
        wp_enqueue_script('genex-detach-restore-script', plugin_dir_url(__FILE__) . 'js/admin.js', array('jquery'), $this->version, true);
        
        wp_localize_script('genex-detach-restore-script', 'genex_detach_restore', array(
            'ajax_url' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('genex_detach_restore_nonce'),
            'backup_text' => __('Backing up product categories...', 'genex-detach-restore'),
            'detach_text' => __('Detaching products from categories...', 'genex-detach-restore'),
            'restore_text' => __('Restoring product categories...', 'genex-detach-restore'),
            'confirm_detach' => __('Are you sure you want to detach all products from their categories? This action can be reversed using the restore function.', 'genex-detach-restore'),
            'confirm_restore' => __('Are you sure you want to restore this backup? This will overwrite current product category assignments.', 'genex-detach-restore')
        ));
    }
    
    // Admin page
    public function admin_page() {
        // Get backups
        $backups = $this->get_backups();
        
        ?>
        <div class="wrap genex-detach-restore">
            <h1><?php echo esc_html__('Detach & Restore Categories', 'genex-detach-restore'); ?></h1>
            
            <div class="card">
                <h2><?php echo esc_html__('Detach Products from Categories', 'genex-detach-restore'); ?></h2>
                <p><?php echo esc_html__('This will remove all products from their assigned categories. A backup will be created automatically.', 'genex-detach-restore'); ?></p>
                <button id="genex-detach-all" class="button button-primary"><?php echo esc_html__('Detach All Products', 'genex-detach-restore'); ?></button>
                
                <div id="genex-progress-container" style="display: none;">
                    <div class="progress-bar-container">
                        <div id="genex-progress-bar" class="progress-bar"></div>
                    </div>
                    <p id="genex-progress-text"></p>
                </div>
                
                <div id="genex-message" class="notice" style="display: none;"></div>
            </div>
            
            <!-- Add Fix Category Counts button -->
            <div class="card">
                <h2><?php echo esc_html__('Fix Category Counts', 'genex-detach-restore'); ?></h2>
                <p><?php echo esc_html__('If your category counts are incorrect, use this button to manually update them.', 'genex-detach-restore'); ?></p>
                <button id="genex-fix-counts" class="button button-secondary"><?php echo esc_html__('Fix Category Counts', 'genex-detach-restore'); ?></button>
                <div id="genex-counts-message" class="notice" style="display: none;"></div>
            </div>
            
            <div class="card">
                <h2><?php echo esc_html__('Restore Previous Backups:', 'genex-detach-restore'); ?></h2>
                
                <table class="widefat striped">
                    <thead>
                        <tr>
                            <th><?php echo esc_html__('Date', 'genex-detach-restore'); ?></th>
                            <th><?php echo esc_html__('Relationships', 'genex-detach-restore'); ?></th>
                            <th><?php echo esc_html__('Actions', 'genex-detach-restore'); ?></th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php if (empty($backups)) : ?>
                            <tr>
                                <td colspan="3"><?php echo esc_html__('No backups available.', 'genex-detach-restore'); ?></td>
                            </tr>
                        <?php else : ?>
                            <?php foreach ($backups as $backup) : ?>
                                <tr>
                                    <td><?php echo esc_html(date_i18n(get_option('date_format') . ' ' . get_option('time_format'), $backup['timestamp'])); ?></td>
                                    <td><?php echo esc_html($backup['count']); ?></td>
                                    <td>
                                        <button class="button genex-restore-backup" data-backup-id="<?php echo esc_attr($backup['id']); ?>">
                                            <?php echo esc_html__('Restore', 'genex-detach-restore'); ?>
                                        </button>
                                    </td>
                                </tr>
                            <?php endforeach; ?>
                        <?php endif; ?>
                    </tbody>
                </table>
            </div>
        </div>
        <?php
    }
    
    // Get backups
    private function get_backups() {
        $backups = get_option('genex_category_backups', array());
        
        // Sort by timestamp (newest first)
        uasort($backups, function($a, $b) {
            return $b['timestamp'] - $a['timestamp'];
        });
        
        return $backups;
    }
    
    // AJAX: Fix category counts
    public function ajax_fix_category_counts() {
        check_ajax_referer('genex_detach_restore_nonce', 'nonce');
        
        if (!current_user_can('manage_woocommerce')) {
            wp_send_json_error(array('message' => __('You do not have permission to perform this action.', 'genex-detach-restore')));
        }
        
        $this->update_term_counts();
        
        wp_send_json_success(array(
            'message' => __('Category counts have been updated successfully.', 'genex-detach-restore')
        ));
    }
    
    // AJAX: Backup category relations
    public function ajax_backup_category_relations() {
        check_ajax_referer('genex_detach_restore_nonce', 'nonce');
        
        if (!current_user_can('manage_woocommerce')) {
            wp_send_json_error(array('message' => __('You do not have permission to perform this action.', 'genex-detach-restore')));
        }
        
        // Get ALL product IDs regardless of categories
        $args = array(
            'post_type' => 'product',
            'posts_per_page' => -1,
            'fields' => 'ids',
            'post_status' => 'any',
        );
        
        $query = new WP_Query($args);
        $product_ids = $query->posts;
        $total = count($product_ids);
        
        if ($total === 0) {
            wp_send_json_error(array('message' => __('No products found.', 'genex-detach-restore')));
        }
        
        // Create backup
        $backup_id = uniqid();
        $backup = array(
            'id' => $backup_id,
            'timestamp' => time(),
            'count' => 0,
            'relations' => array()
        );
        
        // Get all product category terms - use direct SQL for better performance
        global $wpdb;
        $product_id_list = implode(',', array_map('intval', $product_ids));
        $relation_count = 0;
        
        if (!empty($product_id_list)) {
            $query = $wpdb->prepare(
                "SELECT tr.object_id AS product_id, t.term_id, tt.term_taxonomy_id, t.name, t.slug, tt.description, tt.parent
                FROM {$wpdb->term_relationships} AS tr
                INNER JOIN {$wpdb->term_taxonomy} AS tt ON tr.term_taxonomy_id = tt.term_taxonomy_id
                INNER JOIN {$wpdb->terms} AS t ON tt.term_id = t.term_id
                WHERE tr.object_id IN ({$product_id_list})
                AND tt.taxonomy = %s",
                'product_cat'
            );
            
            $results = $wpdb->get_results($query);
            
            foreach ($results as $row) {
                // Skip uncategorized
                if ($row->slug === 'uncategorized') {
                    continue;
                }
                
                $backup['relations'][] = array(
                    'product_id' => (int)$row->product_id,
                    'term_id' => (int)$row->term_id,
                    'term_taxonomy_id' => (int)$row->term_taxonomy_id,
                    'name' => $row->name,
                    'slug' => $row->slug,
                    'description' => $row->description,
                    'parent' => (int)$row->parent
                );
            }
            
            $relation_count = count($backup['relations']);
            $backup['count'] = $relation_count;
        }
        
        // For safety, if no relations found, still check each product individually
        if ($relation_count === 0) {
            foreach ($product_ids as $product_id) {
                $terms = wp_get_object_terms($product_id, 'product_cat', array('fields' => 'all'));
                
                if (!empty($terms) && !is_wp_error($terms)) {
                    // Only count non-uncategorized terms
                    $non_uncategorized_terms = array_filter($terms, function($term) {
                        // Uncategorized in WooCommerce typically has a slug 'uncategorized'
                        return $term->slug !== 'uncategorized';
                    });
                    
                    $relation_count += count($non_uncategorized_terms);
                    
                    foreach ($terms as $term) {
                        // Skip uncategorized category if present
                        if ($term->slug === 'uncategorized') {
                            continue;
                        }
                        
                        $backup['relations'][] = array(
                            'product_id' => $product_id,
                            'term_id' => $term->term_id,
                            'term_taxonomy_id' => $term->term_taxonomy_id,
                            'name' => $term->name,
                            'slug' => $term->slug,
                            'description' => $term->description,
                            'parent' => $term->parent
                        );
                    }
                }
            }
            
            // Update the count in the backup
            $backup['count'] = count($backup['relations']);
        }
        
        // Save backup
        $backups = get_option('genex_category_backups', array());
        $backups[$backup_id] = $backup;
        
        // Keep only the most recent backups
        if (count($backups) > $this->max_backups) {
            // Sort by timestamp (oldest first)
            uasort($backups, function($a, $b) {
                return $a['timestamp'] - $b['timestamp'];
            });
            
            // Remove oldest backups
            $backups = array_slice($backups, count($backups) - $this->max_backups, $this->max_backups, true);
        }
        
        update_option('genex_category_backups', $backups);
        
        wp_send_json_success(array(
            'backup_id' => $backup_id,
            'total_products' => $total,
            'total_relations' => $backup['count'],
            'message' => sprintf(__('Backup completed. Found %d product-category relationships.', 'genex-detach-restore'), $backup['count'])
        ));
    }
    
    // AJAX: Detach products
    public function ajax_detach_products() {
        check_ajax_referer('genex_detach_restore_nonce', 'nonce');
        
        if (!current_user_can('manage_woocommerce')) {
            wp_send_json_error(array('message' => __('You do not have permission to perform this action.', 'genex-detach-restore')));
        }
        
        $backup_id = sanitize_text_field($_POST['backup_id']);
        $offset = isset($_POST['offset']) ? intval($_POST['offset']) : 0;
        $limit = 20; // Process in batches to avoid timeout
        
        // Get product IDs - focus on products that have categories
        $args = array(
            'post_type' => 'product',
            'posts_per_page' => $limit,
            'offset' => $offset,
            'fields' => 'ids',
            'post_status' => 'any',
            'tax_query' => array(
                'relation' => 'OR',
                array(
                    'taxonomy' => 'product_cat',
                    'operator' => 'EXISTS',
                ),
            ),
        );
        
        $query = new WP_Query($args);
        $product_ids = $query->posts;
        
        if (empty($product_ids)) {
            // All products processed, now update term counts
            $this->update_term_counts();
            
            wp_send_json_success(array(
                'done' => true,
                'message' => __('All products have been detached from their categories. Term counts updated.', 'genex-detach-restore')
            ));
        }
        
        // Detach products from categories
        foreach ($product_ids as $product_id) {
            // Get the uncategorized term ID
            $uncategorized = get_term_by('slug', 'uncategorized', 'product_cat');
            
            if ($uncategorized) {
                // Set only to uncategorized
                wp_set_object_terms($product_id, array($uncategorized->term_id), 'product_cat');
            } else {
                // If uncategorized doesn't exist, just remove all categories
                wp_set_object_terms($product_id, array(), 'product_cat');
            }
        }
        
        // Get total number of products for progress calculation
        $total_query = new WP_Query(array(
            'post_type' => 'product',
            'posts_per_page' => -1,
            'fields' => 'ids',
            'post_status' => 'any',
            'tax_query' => array(
                'relation' => 'OR',
                array(
                    'taxonomy' => 'product_cat',
                    'operator' => 'EXISTS',
                ),
            ),
        ));
        
        $total = $total_query->found_posts;
        $processed = $offset + count($product_ids);
        $progress = ($total > 0) ? round(($processed / $total) * 100) : 100;
        
        wp_send_json_success(array(
            'done' => false,
            'offset' => $offset + $limit,
            'total' => $total,
            'processed' => $processed,
            'progress' => $progress,
            'message' => sprintf(__('Detaching products: %d of %d processed (%d%%).', 'genex-detach-restore'), $processed, $total, $progress)
        ));
    }
    
    // Function to update term counts
    private function update_term_counts() {
        global $wpdb;
        
        // First attempt: Use WooCommerce's term recount function
        if (function_exists('wc_recount_terms_by_taxonomy')) {
            wc_recount_terms_by_taxonomy('product_cat');
        } else {
            // Manual update if WooCommerce function isn't available
            // Get all product category terms
            $terms = get_terms(array(
                'taxonomy' => 'product_cat',
                'hide_empty' => false,
            ));
            
            if (empty($terms) || is_wp_error($terms)) {
                return;
            }
            
            foreach ($terms as $term) {
                $count = $wpdb->get_var($wpdb->prepare(
                    "SELECT COUNT(*) FROM $wpdb->term_relationships 
                    JOIN $wpdb->term_taxonomy ON $wpdb->term_relationships.term_taxonomy_id = $wpdb->term_taxonomy.term_taxonomy_id
                    JOIN $wpdb->posts ON $wpdb->term_relationships.object_id = $wpdb->posts.ID
                    WHERE $wpdb->term_taxonomy.term_id = %d 
                    AND $wpdb->term_taxonomy.taxonomy = 'product_cat'
                    AND $wpdb->posts.post_type = 'product'
                    AND $wpdb->posts.post_status = 'publish'",
                    $term->term_id
                ));
                
                $wpdb->update(
                    $wpdb->term_taxonomy,
                    array('count' => $count),
                    array('term_id' => $term->term_id, 'taxonomy' => 'product_cat')
                );
            }
        }
        
        // Clear all caches that could affect term counts
        
        // 1. Clear WordPress term cache
        clean_term_cache(array(), 'product_cat');
        
        // 2. Clear WooCommerce specific caches
        delete_transient('wc_term_counts');
        
        // 3. If available, update WooCommerce's lookup tables
        if (function_exists('wc_clear_template_cache')) {
            wc_clear_template_cache();
        }
        
        // 4. Delete category counts transient
        delete_transient('wc_product_cat_counts');
        
        // 5. Attempt to fire the product category lookup table update
        if (function_exists('wc_update_product_lookup_tables_is_running') && 
            !wc_update_product_lookup_tables_is_running()) {
            if (function_exists('wc_update_product_cat_lookup_table')) {
                wc_update_product_cat_lookup_table();
            } elseif (has_action('wc_update_lookup_table')) {
                do_action('wc_update_lookup_table', 'product_category_lookup');
            }
        }
        
        // 6. General WooCommerce cache flush if available
        if (function_exists('wc_cache_flush')) {
            wc_cache_flush();
        }
        
        // 7. Additionally try to update the product/category relationships
        // in the wc_product_category_lookup table directly if it exists
        $product_category_table_exists = $wpdb->get_var(
            $wpdb->prepare(
                "SHOW TABLES LIKE %s",
                $wpdb->prefix . 'wc_product_category_lookup'
            )
        );
        
        if ($product_category_table_exists) {
            // Clear and rebuild the lookup table
            $wpdb->query("TRUNCATE TABLE {$wpdb->prefix}wc_product_category_lookup");
            
            $wpdb->query("
                INSERT INTO {$wpdb->prefix}wc_product_category_lookup (product_id, category_id)
                SELECT tr.object_id as product_id, tt.term_id as category_id
                FROM {$wpdb->term_relationships} as tr
                INNER JOIN {$wpdb->term_taxonomy} as tt ON tr.term_taxonomy_id = tt.term_taxonomy_id
                INNER JOIN {$wpdb->posts} as posts ON posts.ID = tr.object_id
                WHERE tt.taxonomy = 'product_cat'
                AND posts.post_type = 'product'
                AND posts.post_status = 'publish'
            ");
        }
    }
    
    // AJAX: Restore backup
    public function ajax_restore_backup() {
        check_ajax_referer('genex_detach_restore_nonce', 'nonce');
        
        if (!current_user_can('manage_woocommerce')) {
            wp_send_json_error(array('message' => __('You do not have permission to perform this action.', 'genex-detach-restore')));
        }
        
        $backup_id = sanitize_text_field($_POST['backup_id']);
        $offset = isset($_POST['offset']) ? intval($_POST['offset']) : 0;
        $limit = 20; // Process in batches to avoid timeout
        
        // Get backup
        $backups = get_option('genex_category_backups', array());
        
        if (!isset($backups[$backup_id])) {
            wp_send_json_error(array('message' => __('Backup not found.', 'genex-detach-restore')));
        }
        
        $backup = $backups[$backup_id];
        $total_relations = count($backup['relations']);
        
        if ($offset >= $total_relations) {
            // All relations restored, update term counts
            $this->update_term_counts();
            
            wp_send_json_success(array(
                'done' => true,
                'message' => sprintf(__('Restore completed. %d product-category relationships restored. Term counts updated.', 'genex-detach-restore'), $total_relations)
            ));
        }
        
        // Get relations for this batch
        $batch_relations = array_slice($backup['relations'], $offset, $limit);
        
        // Group relations by product
        $product_relations = array();
        foreach ($batch_relations as $relation) {
            $product_id = $relation['product_id'];
            
            if (!isset($product_relations[$product_id])) {
                $product_relations[$product_id] = array();
            }
            
            $product_relations[$product_id][] = $relation;
        }
        
        // Restore relations
        foreach ($product_relations as $product_id => $relations) {
            $terms = array();
            
            foreach ($relations as $relation) {
                // Check if term exists
                $term = get_term($relation['term_id'], 'product_cat');
                
                if (!$term || is_wp_error($term)) {
                    // Term doesn't exist, recreate it
                    $term_data = array(
                        'description' => $relation['description'],
                        'parent' => $relation['parent'],
                        'slug' => $relation['slug']
                    );
                    
                    $new_term = wp_insert_term($relation['name'], 'product_cat', $term_data);
                    
                    if (!is_wp_error($new_term)) {
                        $terms[] = (int) $new_term['term_id'];
                    }
                } else {
                    $terms[] = (int) $term->term_id;
                }
            }
            
            // Add product to categories
            if (!empty($terms)) {
                wp_set_object_terms($product_id, $terms, 'product_cat');
            }
        }
        
        $processed = $offset + count($batch_relations);
        $progress = ($total_relations > 0) ? round(($processed / $total_relations) * 100) : 100;
        
        wp_send_json_success(array(
            'done' => false,
            'offset' => $offset + $limit,
            'total' => $total_relations,
            'processed' => $processed,
            'progress' => $progress,
            'message' => sprintf(__('Restoring categories: %d of %d relationships processed (%d%%).', 'genex-detach-restore'), $processed, $total_relations, $progress)
        ));
    }
}

// Initialize the plugin
function genex_detach_restore_categories_init() {
    Genex_Detach_Restore_Categories::get_instance();
}
add_action('plugins_loaded', 'genex_detach_restore_categories_init');

// Create CSS directory and file on plugin activation
function genex_detach_restore_activation() {
    // Create CSS directory if it doesn't exist
    $css_dir = plugin_dir_path(__FILE__) . 'css';
    if (!file_exists($css_dir)) {
        wp_mkdir_p($css_dir);
    }
    
    // Create CSS file if it doesn't exist
    $css_file = $css_dir . '/admin.css';
    if (!file_exists($css_file)) {
        $css_content = "
.genex-detach-restore .card {
    padding: 20px;
    background: #fff;
    box-shadow: 0 1px 1px rgba(0,0,0,.04);
    border: 1px solid #c3c4c7;
    margin-bottom: 20px;
}

.genex-detach-restore .progress-bar-container {
    width: 100%;
    background-color: #f1f1f1;
    border-radius: 4px;
    margin: 15px 0;
}

.genex-detach-restore .progress-bar {
    width: 0%;
    height: 20px;
    background-color: #0073aa;
    border-radius: 4px;
    transition: width 0.3s;
}

.genex-detach-restore .notice {
    margin: 15px 0;
    padding: 10px 15px;
}

.genex-detach-restore .notice-success {
    border-left: 4px solid #46b450;
}

.genex-detach-restore .notice-error {
    border-left: 4px solid #dc3232;
}

.genex-detach-restore .notice-warning {
    border-left: 4px solid #ffb900;
}

.genex-detach-restore .notice-info {
    border-left: 4px solid #00a0d2;
}
";
        file_put_contents($css_file, $css_content);
    }
    
    // Create JS directory if it doesn't exist
    $js_dir = plugin_dir_path(__FILE__) . 'js';
    if (!file_exists($js_dir)) {
        wp_mkdir_p($js_dir);
    }
    
    // Create JS file if it doesn't exist
    $js_file = $js_dir . '/admin.js';
    if (!file_exists($js_file)) {
        $js_content = "
jQuery(document).ready(function($) {
    // Fix category counts
    $('#genex-fix-counts').on('click', function() {
        $('#genex-counts-message').hide();
        
        $.ajax({
            url: genex_detach_restore.ajax_url,
            type: 'POST',
            data: {
                action: 'genex_fix_category_counts',
                nonce: genex_detach_restore.nonce
            },
            success: function(response) {
                if (response.success) {
                    $('#genex-counts-message')
                        .removeClass('notice-error')
                        .addClass('notice-success')
                        .html('<p>' + response.data.message + '</p>')
                        .show();
                } else {
                    $('#genex-counts-message')
                        .removeClass('notice-success')
                        .addClass('notice-error')
                        .html('<p>' + response.data.message + '</p>')
                        .show();
                }
            },
            error: function() {
                $('#genex-counts-message')
                    .removeClass('notice-success')
                    .addClass('notice-error')
                    .html('<p>An error occurred while updating category counts.</p>')
                    .show();
            }
        });
    });
    
    // Detach all products
    $('#genex-detach-all').on('click', function() {
        if (!confirm(genex_detach_restore.confirm_detach)) {
            return;
        }
        
        // Show progress container
        $('#genex-progress-container').show();
        $('#genex-progress-text').text(genex_detach_restore.backup_text);
        $('#genex-progress-bar').css('width', '0%');
        $('#genex-message').hide();
        
        // Step 1: Backup category relations
        $.ajax({
            url: genex_detach_restore.ajax_url,
            type: 'POST',
            data: {
                action: 'genex_backup_category_relations',
                nonce: genex_detach_restore.nonce
            },
            success: function(response) {
                if (response.success) {
                    // Step 2: Detach products
                    $('#genex-progress-text').text(genex_detach_restore.detach_text);
                    $('#genex-progress-bar').css('width', '0%');
                    
                    detachProducts(response.data.backup_id, 0);
                } else {
                    showError(response.data.message);
                }
            },
            error: function() {
                showError('An error occurred while backing up categories.');
            }
        });
    });
    
    // Restore backup
    $(document).on('click', '.genex-restore-backup', function() {
        var backupId = $(this).data('backup-id');
        
        if (!confirm(genex_detach_restore.confirm_restore)) {
            return;
        }
        
        // Show progress container
        $('#genex-progress-container').show();
        $('#genex-progress-text').text(genex_detach_restore.restore_text);
        $('#genex-progress-bar').css('width', '0%');
        $('#genex-message').hide();
        
        restoreBackup(backupId, 0);
    });
    
    // Detach products function (processes in batches)
    function detachProducts(backupId, offset) {
        $.ajax({
            url: genex_detach_restore.ajax_url,
            type: 'POST',
            data: {
                action: 'genex_detach_products',
                nonce: genex_detach_restore.nonce,
                backup_id: backupId,
                offset: offset
            },
            success: function(response) {
                if (response.success) {
                    if (response.data.done) {
                        // All products detached
                        $('#genex-progress-bar').css('width', '100%');
                        showSuccess(response.data.message);
                        
                        // Reload page to update backup list
                        setTimeout(function() {
                            location.reload();
                        }, 2000);
                    } else {
                        // Update progress
                        $('#genex-progress-bar').css('width', response.data.progress + '%');
                        $('#genex-progress-text').text(response.data.message);
                        
                        // Process next batch
                        detachProducts(backupId, response.data.offset);
                    }
                } else {
                    showError(response.data.message);
                }
            },
            error: function() {
                showError('An error occurred while detaching products from categories.');
            }
        });
    }
    
    // Restore backup function (processes in batches)
    function restoreBackup(backupId, offset) {
        $.ajax({
            url: genex_detach_restore.ajax_url,
            type: 'POST',
            data: {
                action: 'genex_restore_backup',
                nonce: genex_detach_restore.nonce,
                backup_id: backupId,
                offset: offset
            },
            success: function(response) {
                if (response.success) {
                    if (response.data.done) {
                        // Restore completed
                        $('#genex-progress-bar').css('width', '100%');
                        showSuccess(response.data.message);
                        
                        // Reload page to update backup list
                        setTimeout(function() {
                            location.reload();
                        }, 2000);
                    } else {
                        // Update progress
                        $('#genex-progress-bar').css('width', response.data.progress + '%');
                        $('#genex-progress-text').text(response.data.message);
                        
                        // Process next batch
                        restoreBackup(backupId, response.data.offset);
                    }
                } else {
                    showError(response.data.message);
                }
            },
            error: function() {
                showError('An error occurred while restoring the backup.');
            }
        });
    }
    
    // Show success message
    function showSuccess(message) {
        $('#genex-message')
            .removeClass('notice-error notice-warning notice-info')
            .addClass('notice-success')
            .html('<p>' + message + '</p>')
            .show();
    }
    
    // Show error message
    function showError(message) {
        $('#genex-progress-bar').css('width', '0%');
        $('#genex-message')
            .removeClass('notice-success notice-warning notice-info')
            .addClass('notice-error')
            .html('<p>' + message + '</p>')
            .show();
    }
});
";
        file_put_contents($js_file, $js_content);
    }
}
register_activation_hook(__FILE__, 'genex_detach_restore_activation');